!========================================================================= 
! Soubroutine for calculating phonon frequencies, and eigenvectors
! qvector = phonon wave-vector in reciprocal coordinates.
! frekvens = phonon frequency in [THz]
! eigenvector = antalmod x antalmod matrix, contining the eienvectors.
! The column index of the matrix eigenvector = mode index.
!=========================================================================
SUBROUTINE  getfreq(antalmod,qvector,frekvens,eigenvector)

  USE nrtype
  USE nrutil
  USE diagonalize
  USE data

  IMPLICIT NONE
  
  CHARACTER*1 :: fl
  CHARACTER*2 :: fl1
  CHARACTER*7 :: dum
  LOGICAL :: forsta
  LOGICAL :: finns,linverse, lforceout, lrecip, esiste, equiva1, &
       lgamma = .false.

  INTEGER :: ntyper,nbranches, antalmod, nd, nq, iprint, ndos, j, k, kmax, ndiff, ii
  INTEGER :: i, i1, i2, na, nb, nrr, nr, ntot, idos, ntyp, conta,jj

  INTEGER, ALLOCATABLE :: list(:,:)

  REAL(DP), ALLOCATABLE :: dos(:)

  DOUBLE PRECISION, INTENT(OUT)  :: frekvens(antalmod)

  COMPLEX*16,  INTENT(OUT) :: eigenvector(antalmod,antalmod)

  DOUBLE PRECISION, INTENT(IN) :: qvector(3)
  DOUBLE PRECISION, ALLOCATABLE :: BORNECHARG(:,:,:),EFFCHARG(:)
  DOUBLE PRECISION :: BORN1(3,3),BORN2(3,3),DIELECTR(3,3),DAMPA,PRIMCELLVOL,GG(3),KK(3),COMP1(3),COMP2(3),qfold(3)
  DOUBLE PRECISION :: MATRIXA1(3,3),MATRIXA2(3,3),BORN3(3,3),COMP3(3),COMP4(3),dqfold(3),WECT(3),GCENTER(3)

  REAL(DP) :: temp(3), omegam2, peso, pesotot, delta, slope(3), &
       alpha, a, temperature, q(3), dq(3), zeta, tmp1(3), q0, e, dosin, dosend, &
       dosstep, dossmear, eint, tv, tc, tv0, tc0, omega, fakt, free, &
       free_q, rcut, x, cv, zeropoint, fac
  
  REAL(DP) :: dx, dy, dz

  ! if a frequency is less than -zero is considered a negative (soft) frequency
  REAL(DP), PARAMETER :: zero = 1.d-6, convert_thz_to_cm = 33.357, convert_thz_to_meV = 4.1357    

  TYPE(lattice)  :: latt
  TYPE(shell)    :: sh
  TYPE(dynmat)   :: dyn
  TYPE(symmetry) :: symm
  TYPE(dynmat) :: dcorr
  TYPE(dynmat) :: dcorrgamma
  TYPE(dynmat) :: summaregel
 ! call vtime( tv0, tc0 )
 forsta = .true.
  !==========================================================================
  ! read in run time variables
  !=========================================================================
  call reader( latt, dyn, temperature, lforceout, linverse, alpha, a,  lrecip, sh%rmax, &
       sh%cutoff, dosin, dosend, dosstep, dossmear, &
       nd, iprint, symm%lsymm, dx, dy, dz )

  fac = evtoj   &        !  D^2 U eV  --> Joule
       /1.d-20  &        !  D^2 R A^2 --> meter 
       /amtokg  &        !  proton mass --> Kg
       /1.d24/twopi**2   !  cicles in THZ^2

  ndos = (dosend - dosin)/dosstep + 1
  allocate( dos(ndos) )
  dos = 0

  !======================================================================
  ! set lattice
  !=====================================================================
  call set_lattice ( latt, symm, iprint, sh%rmax,forsta )
  call generate_punti( symm, latt, dx, dy, dz )


  !---------------------------
  ! Number of phonon branches
  !---------------------------
  nbranches = latt%natoms * 3
  
  !===========================================================================================
  ! Reading in dielectric matrix and Born-effective charge matrices and damping parameter DAMPA
  ! from the file DIPOLEINTERACT (if it exsists )
  !============================================================================================
  inquire(file='DIPOLEINTERACT',exist=finns)
  IF (finns) THEN
     ntyper = latt%natoms
     allocate(dcorr%cmat(nbranches,nbranches),BORNECHARG(ntyper,3,3),dcorrgamma%cmat(nbranches,nbranches),EFFCHARG(ntyper))
     allocate(summaregel%cmat(nbranches,nbranches))
     dcorr%cmat(:,:)= (0.0d0,0.0d0)
     OPEN(3489,FILE='DIPOLEINTERACT',STATUS='OLD',ACTION='READ')
     READ(3489,*)DAMPA
     READ(3489,*)PRIMCELLVOL
     READ(3489,*)DIELECTR(1,1),DIELECTR(1,2),DIELECTR(1,3)
     READ(3489,*)DIELECTR(2,1),DIELECTR(2,2),DIELECTR(2,3)
     READ(3489,*)DIELECTR(3,1),DIELECTR(3,2),DIELECTR(3,3)
     DO i=1,ntyper
        READ(3489,*)BORNECHARG(i,1,1),BORNECHARG(i,1,2),BORNECHARG(i,1,3)
        READ(3489,*)BORNECHARG(i,2,1),BORNECHARG(i,2,2),BORNECHARG(i,2,3)
        READ(3489,*)BORNECHARG(i,3,1),BORNECHARG(i,3,2),BORNECHARG(i,3,3)
     ENDDO
     close(3489)
    !DO i=1,ntyper
    ! print*,'----------------------------------------------------------------'
    ! print*,BORNECHARG(i,1,1),BORNECHARG(i,1,2),BORNECHARG(i,1,3)
    ! print*,BORNECHARG(i,2,1),BORNECHARG(i,2,2),BORNECHARG(i,2,3)
    ! print*,BORNECHARG(i,3,1),BORNECHARG(i,3,2),BORNECHARG(i,3,3)
    ! print*,'----------------------------------------------------------------'
    ! print*,DIELECTR(1,1),DIELECTR(1,2),DIELECTR(1,3)
    ! print*,DIELECTR(2,1),DIELECTR(2,2),DIELECTR(2,3)
    ! print*,DIELECTR(3,1),DIELECTR(3,2),DIELECTR(3,3)
    !ENDDO
   ENDIF

  !==================================================
  ! For the inverse power problem, ignore otherwise
  !==================================================
  a = 4*a**alpha
  !---------------------------------------
  ! set the adimensional variable zeta
  !---------------------------------------
  if( dyn%lfree ) then
     zeta = a / exp( alpha * log( latt%volume / latt%natoms ) / 3._dp ) / twopi / ( temperature * bolkev )
     if(linverse)print*,'Zeta = ',zeta
  endif
  !=======================================================================

  !===========================
  ! Build a supercell ?
  !===========================
  if(latt%lsuper)then
     call build_supercell( latt )
     call get_displ( latt, symm, iprint )
     stop
  endif

  !=============================================================================
  ! construct the shell of vectors in real space
  !=============================================================================
  call rshell( sh, latt ) 

  if( linverse )then
!-----------------------------------
! inverse power problem (analytical)
     call force_constant( symm, dyn, sh, latt, a, alpha,forsta )
!-----------------------------------
  else
!---------------------------------------------
! numeric forces (small displacement method)
!---------------------------------------------
     call numeric_force( latt, symm, sh, dyn, iprint )
  endif

  !--------------------------
  ! set the list of vectors
  !--------------------------
  ntot = 0
  do nrr=1,sh%nrm
     do na=1,latt%natoms
        do nb=1,latt%natoms
           tmp1 = sh%r(:,nrr) + latt%x(:,nb) - latt%x(:,na)
           if( vabs(tmp1) < sh%rmax ) ntot = ntot + 1
        enddo
     enddo
  enddo
  allocate( list(3,ntot) )
  ntot = 0
  do nrr=1,sh%nrm
     do na=1,latt%natoms
        do nb=1,latt%natoms
           tmp1 = sh%r(:,nrr) + latt%x(:,nb) - latt%x(:,na)
           if( vabs(tmp1) < sh%rmax ) then
              ntot = ntot + 1
              list(1,ntot) = na
              list(2,ntot) = nb
              list(3,ntot) = nrr
           endif
        enddo
     enddo
  enddo

  !==========================================================
  ! Write the force constant matrix in the file HARMONIC ?
  !==========================================================
  if(lforceout)then
     open(1,file='HARMONIC',status='unknown')
     write(1,*) sh%rmax,'   cutoff radius'
     write(1,*) ntot,'   number of vectors'
     do i = 1, ntot
        na = list(1,i)
        nb = list(2,i)
        nrr = list(3,i)
        write(1,'(''vector:'',4i20)') na, nb, nrr, dyn%weight(na,nb,nrr)
        write(1,'(3f20.12)') sh%r(:,nrr) + latt%x(:,nb) - latt%x(:,na)
        write(1,'(''force constant matrix:'')')
        write(1,'(3f20.12)') dyn%dmat(:,:,na,nb,nrr)
     enddo
     !write(*,'(/''Force constant matrix file HARMONIC written''/)')
  endif

  !==========================================================================
  ! calculate the free energy as the sum of frequencies, you must provide
  ! the file 'QPOINTS' which contains the special points of the BZ 
  ! IN RECIPROCAL LATTICE COORDINATES
  !=========================================================================
  dyn%lfree = .FALSE.
  if( dyn%lfree ) then

     inquire(file='QPOINTS',exist=esiste)
     if(.not.esiste) stop 'cannot find file QPOINTS, use QA, QB and QC to generate one'
     open(10,file='QPOINTS',status='old')

     read(10,*) nd
     !print'(/''Integrating frequencies...'')'
     !print'(/''Using '',i6,'' q-points from file QPOINTS (in reciprocal space coordinates)''/)',nd
     pesotot = 0; omegam2 = 0; eint = 0
     omega = 0; free_q = 0; zeropoint = 0
     free = 0; cv = 0
     conta = 0
     do nq = 1, nd
        read(10,*) q, peso
        pesotot = pesotot + peso
     enddo
     rewind(10)
     read(10,*) nd
     do nq = 1, nd
        read(10,*) q, peso
        q = twopi/latt%scale*matmul(latt%bg,q) ! in cartesian coordinates

        !==========================================================================
        ! construct the dynamical matrix
        !==========================================================================
        call dyn_mat( dyn, latt, sh, q, ntot, list )

        if(iprint>2)then
           print'(''Dynamical matrix:'')'
           do j=1,latt%natoms
              do k=1,latt%natoms
                 print'(2i3)',j,k
                 do i1=1,3
                    print'(6f13.8)', ( dyn%cmat( i1 + (j-1)*3, i2 + (k-1)*3 ), i2=1,3) 
                 enddo
              enddo
           enddo
        endif

        !==========================================================================
        ! diagonalize the dynamical matrix
        !==========================================================================
        call diagh( dyn%cmat, dyn%eig, 'L' )

        if(iprint>2)then
           do j = 1, nbranches
              print'(''Eigenvalue '',i3)',j
              print'(f20.12)',dyn%eig(j) 
              print'(''Eigenvector '',i3)',j
              do k=1,latt%natoms
                 print'(''atom '',i3)',k
                 do i1=1,3
                    print'(2f20.12)', dyn%cmat(i1+(k-1)*3,j)
                 enddo
              enddo
           enddo
        endif

        !=====================
        ! density of states
        !=====================
        do i1 = 1, nbranches
           idos = 0
           kmax = (dosend - dosin)/dosstep
           do k = 1, kmax + 1
              e = dosin + (k-1)*dosstep + dosstep/2
              idos = idos + 1
              if( dyn%eig(i1) >= 0 )then
                 if( sqrt(dyn%eig(i1)) > e - dosstep/2 .and. sqrt(dyn%eig(i1))< e + dosstep/2 )then
                    dos(idos) = dos(idos) + peso
                 endif
              endif
           enddo

           !==========================================================================
           ! the phonon frequencies are the squares of the matrix eigenvalues, for the
           ! inverse power problem I need
           ! the adimensional constants ci = m * omega^2 * V^[(alpha+2)/3] / A (V is
           ! the volume per atom) 
           !==========================================================================
           !-----------------------------------------------------------------------
           ! exclude the acustic mode at gamma 
           !-----------------------------------------------------------------------
           if( dyn%eig(i1) < -zero ) then
              !print*,'negative frequencies',dyn%eig(i1),i1
              !                  stop 'negative frequencies'
              !----------------------------------------------------------
              ! omegam2 is the average logaritmic square frequency.
              !
              ! nu = sqrt( dyn%eig(i) ) in THZ
              ! omega = 2pi * nu
              !
              ! I want m*omega^2*V^(2/3) to be an energy in eV:
              ! 
              ! omega^2 = omega^2 * 1.d24  and now it is in Rad^2 / sec
              !
              ! m = m * amtokg   in Kg
              !
              ! The volume will be expressed in A^3, so I multiply per 1.d-20,
              ! so that V^(3/2) will be in meter^(3/2).
              !
              ! Now m*omega^2*V^(2/3) is in Joule, I divide by 1.6d-19.
              !
              ! 1.d-20 / 1.6d-19 =  0.06241460122_dp
              !  
              !--------------------------------------------------------------

           elseif( dyn%eig(i1) > zero ) then

              conta = conta + 1
              fakt = 1.d4 * twopi**2 / evtoj 

              omega = omega + peso * log( dyn%eig(i1) * fakt )

              free = free + peso * bolkev * temperature * &
                   log( hplank * sqrt( dyn%eig(i1) )*1.d12 / temperature / bolk )

              free_q = free_q  + peso * ( &
                   0.5_dp * hplank * sqrt( dyn%eig(i1) )*1.d12 / evtoj + &
                   bolkev * temperature * &
                   log( 1 - exp( -hplank * sqrt( dyn%eig(i1) )*1.d12 / temperature / bolk) ) )

              zeropoint = zeropoint + peso * &
                   0.5_dp * hplank * sqrt( dyn%eig(i1) )*1.d12 / evtoj

              omegam2 = omegam2 + peso * ( log( dyn%eig(i1)*fakt ) + log( latt%mass(1)*amtokg ) )

              eint = eint + peso * (hplank/2)*sqrt( dyn%eig(i1) )*1.d12 * &
                   coth( hplank*sqrt( dyn%eig(i1) )*1.d12 / &
                   ( 2*bolk*Temperature ) ) 
              
              x = hplank*sqrt( dyn%eig(i1) )*1.d12 / ( bolk*Temperature ) 
              cv = cv + peso * x**2*exp(x)/(exp(x) - 1)**2

           endif
        enddo
     enddo

     omegam2 = omegam2 / pesotot / nbranches
     omega = omega / pesotot / nbranches
     free = free / pesotot
     free_q = free_q / pesotot
     zeropoint = zeropoint / pesotot
     cv = cv / pesotot
     eint = eint / pesotot / evtoj

     if( conta /= nbranches*nd ) &
          write(*,'(/''WARNING, Found '',i8, &
          & '' frequencies different from zero, out of a total of '',i8/)') conta, nbranches*nd

     !---------------------------------------------------------------------------
     ! print out (excess) free energy(classical limit) per atom for the solid
     !---------------------------------------------------------------------------
     write(6,'(/''Your primitive cell contains N ='',i3,'' atoms''/)') latt%natoms
     write(6,'(''Temperature  '',f10.2,'' K'')') temperature
     write(6,'(''Zero point energy           '',f15.5,''  (eV/cell)'')') zeropoint
     write(6,'(''Free energy                 '',f15.5,''  (eV/cell)'', &
          & f12.5,''  (eV/atom)'')') free_q, free_q/latt%natoms 
     write(6,'(''Free energy(classical_limit)'',f15.5,''  (eV/cell)'', &
          & f12.5,''  (eV/atom)'')') free, free/latt%natoms
     write(6,'(''Internal Energy             '',f15.5,''  (eV/cell)'')') eint
     write(6,'(''3 N KT                      '',f15.5,''  (eV/cell)'')') &
          3*temperature*bolkev*latt%natoms
     write(6,'(''Cv                          '',f15.5,''  (K/cell)'')') cv
     write(6,'(''S                           '',f15.5,''  (K/cell)''/)') &
          (eint - free_q) / (temperature*bolkev)

     write(6,'(''Omegam2:'',f15.5)') omegam2

     open(4,file='DOS',status='unknown')
     open(44,file='DOS.cm',status='unknown')
     open(45,file='DOS.meV',status='unknown')
     idos = 0
     dos = dos / pesotot / nbranches / dosstep
     call smooth( dos, dossmear, ndos, dosin, dosstep )
     !write(6,'(/''Density of states integral = '',f10.5)') sum(dos)*dosstep
     if( abs(sum(dos)*dosstep-1.0_dp) > 1.d-6 ) then
        !print*,'renormalizing DOS..'
        dos = dos / ( sum(dos)*dosstep )
     endif        

     kmax = (dosend - dosin)/dosstep
     do k = 1, kmax + 1
        e = dosin + (k-1)*dosstep + dosstep/2
        idos = idos + 1
        write(4,'(2f20.5)') e, dos(idos)
        write(44,'(2f20.5)') e*convert_thz_to_cm, dos(idos)/convert_thz_to_cm
        write(45,'(2f20.5)') e*convert_thz_to_mev, dos(idos)/convert_thz_to_mev
     enddo

   !  call vtime( tv, tc )
   ! write(*,'(/''CPU time '',f10.2,10x,''Elapsed time '',f10.2)') tv - tv0, tc - tc0

     stop

  endif

  !============================================================================
  ! calculate phonon dispersions
  !===========================================================================

  do i=1,nbranches/3
     if( i <=9 ) then
        write(fl,'(i1)')i
        open(10+i,file='FREQ.LO_TO'//fl,status='unknown')
     else
        write(fl1,'(i2)')i
        open(10+i,file='FREQ.LO_TO'//fl1,status='unknown')
     endif
  enddo
  open(2,file='FREQ.LO_TO',status='unknown')
  open(3,file='FREQ.cm',status='unknown')
  q0=0
  !write(*,'(/''Distances:'')')
  do nq = 1, nd
     !print'(''point '',i4,f10.6,10x,3f10.6)', nq, q0, dyn%qi(:,nq)
     if( dyn%npoints > 1 ) then
        dq = ( dyn%qf(:,nq) - dyn%qi(:,nq) ) / ( dyn%npoints - 1 )      
     else
        dq = 0
     endif
     dqfold = dq
     if( lrecip ) dq = matmul(latt%bg,dq)
     q0 = q0 - vabs( dq )
     do i = 1, dyn%npoints
        q = dyn%qi(:,nq)
        qfold = dyn%qi(:,nq)+(i-1)*dqfold
       GCENTER = 0.0d0 
       if (DABS(qfold(1)) .GT. 0.5 .AND. lrecip ) THEN 
           IF ( qfold(1) < 0.0d0 ) THEN
              qfold(1) =qfold(1)+1.0d0
              GCENTER(1) = -1.0d0
           ELSE
              qfold(1) =qfold(1)-1.0d0
              GCENTER(1) = 1.0d0
           ENDIF
        ENDIF
        if (DABS(qfold(2)) .GT. 0.5 .AND. lrecip ) THEN 
          IF ( qfold(2) < 0.0d0 ) THEN
            qfold(2) =qfold(2)+1.0d0
            GCENTER(2) = -1.0d0
          ELSE
              qfold(2) =qfold(2)-1.0d0
              GCENTER(2) = 1.0d0
          ENDIF
        ENDIF
        if (DABS(qfold(3)) .GT. 0.5 .AND. lrecip ) THEN
          IF ( qfold(3) < 0.0d0 ) THEN           
             qfold(3) =qfold(3)+1.0d0
             GCENTER(3) = -1.0d0
          ELSE
              qfold(3) =qfold(3)-1.0d0
              GCENTER(3) = 1.0d0
          ENDIF
        ENDIF
        if( lrecip ) then
           q = twopi/latt%scale*matmul(latt%bg,q) + &
                (i-1)*dq*twopi/latt%scale
           qfold = twopi/latt%scale*matmul(latt%bg,qfold)
           GCENTER = twopi/latt%scale*matmul(latt%bg,GCENTER)
        else
           q = twopi/latt%scale*q + (i-1)*dq*twopi/latt%scale
           qfold = twopi/latt%scale*qfold
           GCENTER = twopi/latt%scale*GCENTER 
        endif
        if(iprint > 1)then
           write(*,'(/''Point #'',i3,'' q-vector: '',3f10.5)') i, q*latt%scale/twopi
        endif
        q0 = q0 + vabs(dq)

        !==========================================================================
        ! construct the dynamical matrix
        !==========================================================================
        if ( i .GT. 1 ) DEALLOCATE(dyn%cmat,dyn%eig)
        call dyn_mat( dyn, latt, sh, q, ntot, list )

        if(iprint>2)then
           print'(''Dynamical matrix:'')'
           do j=1,latt%natoms
              do k=1,latt%natoms
                 print'(2i3)',j,k
                 do i1=1,3
                    print'(6f13.8)', &
                         ( dyn%cmat( i1 + (j-1)*3, i2 + (k-1)*3 )/fac, i2=1,3) 
                 enddo
              enddo
           enddo
        endif
        !============================================================================================
        ! Calculate the longrange dipole-dipole interaction correction term for 
        ! for q-points around the gamma point. This correction is zero at all 
        ! commensurate points. A more accurate longrange dipole correction 
        ! should in principle be present in all non-commensurate dynamical matrices
        ! but here we choose to correct only the q-points close to the Gamma pont.
        ! More explicitly we use the correction of Parlinski et al, Phys. Rev. Lett. 81, 3298 (1998)
        !============================================================================================

        IF (finns) THEN
           dcorr%cmat(:,:) = (0.0d0,0.0d0)
           DO na=1,ntyper
              DO nb=1,ntyper
                 BORN1(:,:) = BORNECHARG(na,:,:)
                 BORN2(:,:) = BORNECHARG(nb,:,:)
                 COMP1 = MATMUL(BORN1,qfold)
                 COMP2 = MATMUL(BORN2,qfold)
                 MATRIXA1(1,:) = COMP1(1)*COMP2
                 MATRIXA1(2,:) = COMP1(2)*COMP2
                 MATRIXA1(3,:) = COMP1(3)*COMP2
                 IF ( dot_product(qfold,MATMUL(DIELECTR,qfold)) .NE. 0.0d0 ) THEN
                      dcorr%cmat( 1 + (na-1)*3 : 3 + (na-1)*3, 1 + (nb-1)*3 : 3 + (nb-1)*3 ) = &
                      (MATRIXA1(:,:)/( dot_product(qfold,MATMUL(DIELECTR,qfold)) ))*exp( -dot_product(qfold,qfold)/(DAMPA**2) ) &
                      *exp(-im*dot_product(latt%x(:,na) - latt%x(:,nb),GCENTER))/sqrt( latt%mass( latt%ityp(na) ) * latt%mass(latt%ityp(nb)) )
                 ENDIF
                 IF ( dot_product(qfold,qfold) .EQ. 0.0d0 ) THEN
                    IF ( dot_product(dq,dq) .NE. 0.0d0 ) THEN
                        KK = dq*twopi/latt%scale
                    ELSE
                        KK(1) = 1.0d0
                        KK(2) = 0.0d0
                        KK(3) = 0.0d0
                    ENDIF
                    COMP1 = MATMUL(KK,BORN1)
                    COMP2 = MATMUL(KK,BORN2)
                    MATRIXA1(1,:) = COMP1(1)*COMP2
                    MATRIXA1(2,:) = COMP1(2)*COMP2
                    MATRIXA1(3,:) = COMP1(3)*COMP2                    
                    dcorr%cmat( 1 + (na-1)*3 : 3 + (na-1)*3, 1 + (nb-1)*3 : 3 + (nb-1)*3 ) = &
                      (MATRIXA1(:,:)/( dot_product(KK,MATMUL(DIELECTR,KK)))) &
                      *exp(-im*dot_product(latt%x(:,na) - latt%x(:,nb),GCENTER))/sqrt( latt%mass( latt%ityp(na) ) * latt%mass(latt%ityp(nb)) )
                 ENDIF
              ENDDO
             ENDDO
             ! Fixing the right units
             dcorr%cmat = dcorr%cmat*2*twopi/(2*twopi*PRIMCELLVOL)!
             dcorr%cmat = dcorr%cmat*(1602.0/8.8541878)  ! ---> eV/(atomic mass unit * angstroem**2)
             dcorr%cmat = dcorr%cmat*evtoj/1.d-20/amtokg/1.d24/twopi**2 ! ----> THZ^2
             IF (dot_product(q,q) .EQ.  0.0d0 ) dcorrgamma%cmat = dcorr%cmat
             dyn%cmat = dyn%cmat + dcorr%cmat
             !IF (dot_product(q,q) .EQ.  0.0d0 ) THEN
             !   print*,q
             !   print*,GCENTER
             !   print*,KK
             !   DO na=1,ntyper
             !     DO nb=1,ntyper
             !        print*,'-----',na,nb,'-------------------'
             !        print*, dcorr%cmat( 1 + (na-1)*3, 1 + (nb-1)*3 ),dcorr%cmat( 1 + (na-1)*3, 2 + (nb-1)*3 ),dcorr%cmat( 1 + (na-1)*3, 3 + (nb-1)*3 )
             !        print*, dcorr%cmat( 2 + (na-1)*3, 1 + (nb-1)*3 ),dcorr%cmat( 2 + (na-1)*3, 2 + (nb-1)*3 ),dcorr%cmat( 2 + (na-1)*3, 3 + (nb-1)*3 )
             !        print*, dcorr%cmat( 3 + (na-1)*3, 1 + (nb-1)*3 ),dcorr%cmat( 3 + (na-1)*3, 2 + (nb-1)*3 ),dcorr%cmat( 3 + (na-1)*3, 3 + (nb-1)*3 )
             !        print*,'------------------------------------'
             !     ENDDO
             !  ENDDO
             !  STOP
             ! ENDIF
          ENDIF
        !==========================================================================
        ! diagonalize the dynamical matrix
        !==========================================================================
        call diagh( dyn%cmat, dyn%eig, 'L' )


        do i1=1,nbranches

           !==========================================================================
           ! the phonon frequencies are the square roots of the matrix eigenvalues
           !==========================================================================
           if( dyn%eig(i1) < 0 ) then
              dyn%eig(i1) = -sqrt(-dyn%eig(i1))
           else
              dyn%eig(i1) = sqrt(dyn%eig(i1))
           endif
        enddo
        write(2,'(f12.8,42f12.5)')q0,dyn%eig
        write(3,'(f12.8,42f12.5)')q0,dyn%eig*convert_thz_to_cm
        do i1=1,nbranches/3
           write(10+i1,'(4f12.5)')q0,dyn%eig( 1 + (i1 - 1)*3 : i1*3 )
        enddo

        !--------------------------------------------------
        ! estimates the slopes of the first three branches
        !--------------------------------------------------
        if( iprint > 0 ) then
           if( i == 1 .or. i == dyn%npoints - 1 ) then
              slope = dyn%eig(1:3)
           elseif( i == 2 .or. i == dyn%npoints ) then
              slope = ( dyn%eig(1:3) - slope ) / vabs( dq/latt%scale ) * 100
              write(*,'(''Slopes (m/s) : '',3f10.1)') slope
           endif
        endif

        !=====================
        ! density of states
        !=====================
        do i1 = 1, nbranches
           idos = 0
           do k = 1, ndos
              e = dosin + (k-1)*dosstep
              idos = idos + 1
              if( dyn%eig(i1) >= 0 )then
                 if( dyn%eig(i1) > e - dosstep/2 .and. dyn%eig(i1) < e + dosstep/2 )then
                    dos(idos) = dos(idos) + 1
                 endif
              endif
           enddo
        enddo

        if(iprint>2)then
           do j=1,nbranches
              Print'(''Eigenvalue '',i3,'' : '',f12.5)',j,dyn%eig(j) 
              print'(''Eigenvector '',i3)',j
              do k=1,latt%natoms
                 print'(''atom '',i3)',k
                 do i1=1,3
                    print'(2f10.4)', dyn%cmat(i1+(k-1)*3,j)
                 enddo
              enddo
           enddo
        endif
     enddo
  enddo

  !print'(''end point '',f10.6,10x,3f10.6)',  q0, dyn%qf(:,nd)

  open(4,file='DOS',status='unknown')
  open(44,file='DOS.cm',status='unknown')
  open(45,file='DOS.meV',status='unknown')
  idos = 0
  dos = dos / nd / dyn%npoints / nbranches / dosstep
  call smooth( dos, dossmear, ndos, dosin, dosstep )
  !write(6,'(/''Density of states integral = '',f10.5)') sum(dos)*dosstep
  if( abs(sum(dos)*dosstep-1.0_dp) > 1.d-6 ) then
     !print*,'renormalizing DOS..'
     dos = dos / ( sum(dos)*dosstep )
  endif
  do k = 1, ndos
     e = dosin + (k-1)*dosstep
     idos = idos + 1
     write(4,'(2f20.5)') e, dos(idos)
     write(44,'(2f20.5)') e*convert_thz_to_cm, dos(idos)/convert_thz_to_cm
     write(45,'(2f20.5)') e*convert_thz_to_meV, dos(idos)/convert_thz_to_mev
  enddo

  !call vtime( tv, tc )
  !write(*,'(/''CPU time '',f10.2,10x,''Elapsed time '',f10.2)') tv - tv0, tc - tc0
  

  !------------------------------------------
  ! Here the eigenvectors and eigenvalues of qvector are calculated
  !-----------------------------------------
  q = qvector(:)
  q = twopi/latt%scale*matmul(latt%bg,q) ! in cartesian coordinates
  DEALLOCATE(dyn%cmat,dyn%eig)
  call dyn_mat( dyn, latt, sh, q, ntot, list ) ! Dynamical matrix D(q) 
  
  ! Here only for the gamma point the LO-TO correction is added.
  ! That is if the 
  IF ( dot_product(q,q) .EQ.  0.0d0  .AND. finns ) dyn%cmat = dyn%cmat + dcorrgamma%cmat
   
  call diagh( dyn%cmat, dyn%eig, 'L' )

  DO j=1,nbranches
        eigenvector(:,j) = dyn%cmat(:,j)
        if( dyn%eig(j) < 0 ) then
                frekvens(j) = -sqrt(-dyn%eig(j))
        else
                frekvens(j) = sqrt(dyn%eig(j))
        endif
  enddo
        if(dot_product(q,q) .LT. 0.0000001 )then
           do j = 1, nbranches
              print'(''Eigenvalue '',i3)',j
              print'(f20.12)',dyn%eig(j)
              print'(''Eigenvector '',i3)',j
              do k=1,latt%natoms
                 print'(''atom '',i3)',k
                 do i1=1,3
                    print'(2f20.12)', dyn%cmat(i1+(k-1)*3,j)
                 enddo
              enddo
           enddo
        endif
if(lforceout) close(1)
close(2)
close(3)
close(4)
close(44)
close(45)

DEALLOCATE(dyn%cmat,dyn%eig)
IF (finns) DEALLOCATE(dcorr%cmat,BORNECHARG,dcorrgamma%cmat,EFFCHARG,summaregel%cmat)
IF ( .not. forsta ) THEN
                deallocate(symm%is,latt%ityp,latt%x,latt%prim_atom,symm%irt,symm%irts)
                deallocate( symm%iscryst,symm%isstart,symm%ftau)
ENDIF
deallocate ( latt%xs,latt%ityps)
deallocate( latt%nions, latt%mass)
deallocate(sh%r, sh%rr,sh%rl)
deallocate(dyn%dmat,dyn%weight)
if ( nd .GT. 0 ) deallocate( dyn%qi, dyn%qf)
END SUBROUTINE  getfreq

